/*
 * $Header: /home/cvspublic/jakarta-commons-sandbox/resources/src/java/org/apache/commons/resources/impl/ResourcesBase.java,v 1.6 2003/08/17 03:34:14 craigmcc Exp $
 * $Revision: 1.6 $
 * $Date: 2003/08/17 03:34:14 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Commons", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */


package org.apache.commons.resources.impl;


import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;
import java.util.Iterator;
import java.util.Locale;
import java.util.TimeZone;

import org.apache.commons.resources.Resources;
import org.apache.commons.resources.ResourcesException;


/**
 * <p>Convenience base class for {@link Resources} implementations.</p>
 *
 * <p>Default implementations of the content retrieval methods are provided
 * for all methods except <code>getObject()</code>.  The default methods for
 * the other content retrieval methods are coded in terms of
 * <code>getString()</code>, on the assumption that most uses of
 * {@link Resources} are of this type.  However, they can be easily
 * overridden as needed.</p>
 *
 * @author Mike Schachter (mschachter@hp.com)
 * @author Craig R. McClanahan
 * @version $Revision: 1.6 $ $Date: 2003/08/17 03:34:14 $
 */

public abstract class ResourcesBase implements Resources {


    // ----------------------------------------------------------- Constructors


    /**
     * <p>Create a new {@link Resources} instance with no name.</p>
     */
    public ResourcesBase() {

        this(null);

    }


    /**
     * <p>Create a new {@link Resources} instance with the specified
     * logical name.</p>
     *
     * @param name Logical name of the new instance
     */
    public ResourcesBase(String name) {

        this.name = name;

    }


    // ----------------------------------------------------- Instance Variables


    /**
     * <p>The logical name of this {@link Resources} instance.</p>
     */
    private String name = null;


    /**
     * <p>Flag indicating whether resource getter methods should return
     * <code>null</code> (instead of throwing an exception) on invalid
     * key values.</p>
     */
    private boolean returnNull = true;


    // ------------------------------------------------------ Lifecycle Methods


    /**
     * <p>This must be called to initialize the data content of this
     * {@link Resources} instance, before any of the <code>getXxx()</code>
     * methods are called.</p>
     *
     * <p>The default implementation does nothing.</p>
     *
     * @exception ResourcesException if an error occurs during initialization
     */
    public void init() throws ResourcesException {

        ; // The default implementation does nothing

    }


    /**
     * <p>This method must be called when the manager of this resource
     * decides that it's no longer needed.  After this method is called,
     * no further calls to any of the <code>getXxx()</code> methods are
     * allowed.</p>
     *
     * <p>The default implementation does nothing.</p>
     *
     * @exception ResourcesException if an error occurs during finalization
     */
    public void destroy() throws ResourcesException {

        ; // The default implementation does nothing

    }


    // ------------------------------------------------------------- Properties


    public abstract Iterator getKeys();


    /**
     * <p>Return the logical name of this {@link Resources} instance.</p>
     */
    public String getName() {

        return (this.name);

    }


    /**
     * <p>Return <code>true</code> if resource getter methods will return
     * <code>null</code> instead of throwing an exception on invalid
     * key values.</p>
     */
    public boolean isReturnNull() {

        return (this.returnNull);

    }


    /**
     * <p>Set a flag determining whether resource getter methods should
     * return <code>null</code> instead of throwing an exception on
     * invalid key values.</p>
     *
     * @param returnNull The new flag value
     */
    public void setReturnNull(boolean returnNull) {

        this.returnNull = returnNull;

    }


    // ---------------------------------------------- Content Retrieval Methods


    /**
     * <p>Return the content for the specified <code>key</code> as a
     * byte array, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * <p>The default implementation calls <code>getString()</code> and
     * converts the value to a byte array.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public byte[] getBytes(String key, Locale locale, TimeZone timeZone) {

        String value = getString(key, locale, timeZone);
        if (value == null) {
            return (null);
        } else {
            return (value.getBytes());
        }

    }


    /**
     * <p>Return the content for the specified <code>key</code> as an
     * InputStream, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * <p>The default implementation calls <code>getsBytes()</code>
     * and returns an input stream over the resulting byte array.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public InputStream getInputStream(String key, Locale locale,
                                      TimeZone timeZone) {

        byte bytes[] = getBytes(key, locale, timeZone);
        if (bytes == null) {
            bytes = new byte[0];
        }
        return (new ByteArrayInputStream(bytes));

    }


    /**
     * <p>Return the content for the specified <code>key</code> as an
     * Object, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * <p>There is no default implementation of this method.  Concrete
     * subclasses must provide such an implementation.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public abstract Object getObject(String key, Locale locale, TimeZone timeZone);


    /**
     * <p>Return the content for the specified <code>key</code> as a
     * Reader, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * <p>The default implementation calls <code>getString()</code>
     * and returns a reader over the resulting characters.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public Reader getReader(String key, Locale locale, TimeZone timeZone) {

        String value = getString(key, locale, timeZone);
        if (value == null) {
            return (null);
        } else {
            return (new StringReader(value));
        }

    }


    /**
     * <p>Return the content for the specified <code>key</code> as a
     * String, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * <p>The default implementation calls <code>getObject()</code>
     * and converts the result to a String if necessary.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public String getString(String key, Locale locale, TimeZone timeZone) {

        Object value = getObject(key, locale, timeZone);
        if (value == null) {
            return ((String) null);
        } else if (value instanceof String) {
            return ((String) value);
        } else {
            return (value.toString());
        }

    }

}
